VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsDirectDraw"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'********************************************
'* clsDirectDraw.cls                        *
'* Version 1.1                              *
'* Core DirectDraw7 engine class            *
'********************************************
Option Explicit

'DirectX variables
Public DirectX As DirectX7
Public DirectDraw As DirectDraw7

'Primary surface variables
Public PrimarySurface As DirectDrawSurface7
Dim PrimDesc As DDSURFACEDESC2
Dim PrimRect As DxVBLib.RECT
Dim Clipper As DirectDrawClipper

'Double buffer variables
Public DoubleBuffer As DirectDrawSurface7
Dim DoubleBufferDesc As DDSURFACEDESC2
Dim DoubleBufferCaps As DDSCAPS2
Dim DoubleBufferRect As DxVBLib.RECT      'RECT used to encompass the whole doublebuffer
Dim BlittingRect As DxVBLib.RECT    'RECT used when blitting to the double buffer. Can be modified by user

'Class variables
Public TargetDevice As Long   'variable to hold the handle to a destination drawing device other then the default specified
Dim CurrentHandle As Long       'The handle to the device that we draw on
Dim WindowHandle As Long        'Handle to the actual form
Dim FullScreenDevice As Boolean

'******************************
'Class Initialize and terminate
'******************************
Private Sub Class_Terminate()
    Set Clipper = Nothing
    Set DoubleBuffer = Nothing
    Set PrimarySurface = Nothing
    Set DirectDraw = Nothing
    Set DirectX = Nothing
End Sub

'*****************************
'Class methods
'*****************************
Public Function Initialize(hWnd As Long, Optional Fullscreen As Boolean = False, Optional Width As Long, Optional Height As Long, Optional BitDepth As Long) As Long
    On Error GoTo init_error
    
    Err.Clear
    FullScreenDevice = Fullscreen
    WindowHandle = hWnd
    'Reset everything
    Set Clipper = Nothing
    Set DoubleBuffer = Nothing
    Set PrimarySurface = Nothing
    Set DirectDraw = Nothing
    Set DirectX = Nothing
    'First initialize DirectX
    Set DirectX = New DirectX7
    Set DirectDraw = DirectX.DirectDrawCreate("")
    
    If Fullscreen Then
        DirectDraw.SetCooperativeLevel hWnd, DDSCL_FULLSCREEN Or DDSCL_EXCLUSIVE
        DirectDraw.SetDisplayMode Width, Height, BitDepth, 0, DDSDM_DEFAULT
    Else
        DirectDraw.SetCooperativeLevel hWnd, DDSCL_NORMAL
    End If
    
    'Now create the primary surface...
    'Create descriptions depending on fullscreen or windowed mode
    If Fullscreen Then
        PrimDesc.lFlags = DDSD_CAPS Or DDSD_BACKBUFFERCOUNT
        PrimDesc.ddsCaps.lCaps = DDSCAPS_PRIMARYSURFACE Or DDSCAPS_FLIP Or DDSCAPS_COMPLEX
        PrimDesc.lBackBufferCount = 1
    Else
        PrimDesc.lFlags = DDSD_CAPS
        PrimDesc.ddsCaps.lCaps = DDSCAPS_PRIMARYSURFACE
    End If
    
    Set PrimarySurface = DirectDraw.CreateSurface(PrimDesc)
    If PrimarySurface Is Nothing Then GoTo init_error
    PrimarySurface.GetSurfaceDesc PrimDesc
    
    'Get the dimensions of the displaying device
    If TargetDevice Then
        CurrentHandle = TargetDevice
    Else
        CurrentHandle = hWnd
    End If

    If Not Fullscreen Then
        'Create the clipper object
        Set Clipper = DirectDraw.CreateClipper(0)
        Clipper.SetHWnd CurrentHandle
        PrimarySurface.SetClipper Clipper
    End If
    
    DirectX.GetWindowRect CurrentHandle, PrimRect
    
    '...and then create the back buffer
    
    If Fullscreen Then
        DoubleBufferCaps.lCaps = DDSCAPS_BACKBUFFER
        Set DoubleBuffer = PrimarySurface.GetAttachedSurface(DoubleBufferCaps)
    Else
        DirectX.GetWindowRect CurrentHandle, DoubleBufferRect
        DoubleBufferDesc.lFlags = DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH
        DoubleBufferDesc.ddsCaps.lCaps = DDSCAPS_OFFSCREENPLAIN
        If Not (Width And Height) Then
            DoubleBufferDesc.lWidth = DoubleBufferRect.Right - DoubleBufferRect.Left
            DoubleBufferDesc.lHeight = DoubleBufferRect.Bottom - DoubleBufferRect.Top
        Else
            DoubleBufferDesc.lWidth = Width
            DoubleBufferDesc.lHeight = Height
        End If
        Set DoubleBuffer = DirectDraw.CreateSurface(DoubleBufferDesc)
    End If
    
    DoubleBuffer.GetSurfaceDesc DoubleBufferDesc
    If DoubleBuffer Is Nothing Then GoTo init_error
    
    With DoubleBufferRect
        .Top = 0
        .Left = 0
        .Right = DoubleBufferDesc.lWidth
        .Bottom = DoubleBufferDesc.lHeight
    End With
    
    BlittingRect = DoubleBufferRect
    
    Initialize = 0
    Exit Function
init_error:
    Initialize = Err.Number
End Function

Public Function Present() As Long
    On Error GoTo present_error
    
    Err.Clear
    If FullScreenDevice Then
        PrimarySurface.Flip Nothing, DDFLIP_WAIT
        Present = 0
    Else
        DirectX.GetWindowRect CurrentHandle, PrimRect
        Present = PrimarySurface.Blt(PrimRect, DoubleBuffer, DoubleBufferRect, DDBLT_WAIT)
    End If
    Exit Function
present_error:
    Present = Err.Number
End Function

Public Function FillScreen(Colour As Long) As Long
    On Error GoTo fill_error
    Err.Clear
    
    FillScreen = DoubleBuffer.BltColorFill(DoubleBufferRect, Colour)
    Exit Function
fill_error:
    FillScreen = Err.Number
End Function

'******************************
'Blitting Functions
'******************************
'Surface Blitting Functions
Public Function SurfaceBlt(ByRef Surface As DirectDrawSurface7, SrcRect As DxVBLib.RECT, Optional Transparent As Boolean = False) As Long
    On Error GoTo blt_error
    Err.Clear
    
    If Transparent Then
        SurfaceBlt = DoubleBuffer.Blt(BlittingRect, Surface, SrcRect, DDBLT_WAIT Or DDBLT_KEYSRC)
    Else
        SurfaceBlt = DoubleBuffer.Blt(BlittingRect, Surface, SrcRect, DDBLT_WAIT)
    End If
    Exit Function
blt_error:
    SurfaceBlt = Err.Number
End Function

Public Function SurfaceBltFast(ByRef Surface As DirectDrawSurface7, SrcRect As DxVBLib.RECT, XPos As Long, YPos As Long, Optional Transparent As Boolean = False) As Long
    On Error GoTo blt_error
    Err.Clear
    If Transparent Then
        SurfaceBltFast = DoubleBuffer.BltFast(XPos, YPos, Surface, SrcRect, DDBLTFAST_WAIT Or DDBLTFAST_SRCCOLORKEY)
    Else
        SurfaceBltFast = DoubleBuffer.BltFast(XPos, YPos, Surface, SrcRect, DDBLTFAST_WAIT)
    End If
    Exit Function
blt_error:
    SurfaceBltFast = Err.Number
End Function

'Bitmap blitting functions
Public Function BitmapBlt(ByRef Bitmap As clsBitmap) As Long
    On Error GoTo blt_error
    Err.Clear
    If Bitmap.TransparencyEnabled Then
        BitmapBlt = DoubleBuffer.Blt(BlittingRect, Bitmap.Surface, Bitmap.Rectangle, DDBLT_WAIT Or DDBLT_KEYSRC)
    Else
        BitmapBlt = DoubleBuffer.Blt(BlittingRect, Bitmap.Surface, Bitmap.Rectangle, DDBLT_WAIT)
    End If
    Exit Function
blt_error:
    BitmapBlt = Err.Number
End Function

Public Function BitmapBltFast(ByRef Bitmap As clsBitmap) As Long
    On Error GoTo blt_error
    Err.Clear
    If Bitmap.TransparencyEnabled Then
        BitmapBltFast = DoubleBuffer.BltFast(Bitmap.X, Bitmap.Y, Bitmap.Surface, Bitmap.Rectangle, DDBLTFAST_WAIT Or DDBLTFAST_SRCCOLORKEY)
    Else
        BitmapBltFast = DoubleBuffer.BltFast(Bitmap.X, Bitmap.Y, Bitmap.Surface, Bitmap.Rectangle, DDBLTFAST_WAIT)
    End If
blt_error:
    BitmapBltFast = Err.Number
End Function

'Sprite blitting routines
Public Function SpriteBlt(ByRef Sprite As clsSprite) As Long
    On Error GoTo blt_error
    Err.Clear
    If Sprite.TransparencyEnabled Then
        SpriteBlt = DoubleBuffer.Blt(BlittingRect, Sprite.Surface, Sprite.FrameRect(Sprite.CurrentFrame), DDBLT_WAIT Or DDBLT_KEYSRC)
    Else
        SpriteBlt = DoubleBuffer.Blt(BlittingRect, Sprite.Surface, Sprite.FrameRect(Sprite.CurrentFrame), DDBLT_WAIT)
    End If
    Exit Function
blt_error:
    SpriteBlt = Err.Number
End Function

Public Function SpriteBltFast(ByRef Sprite As clsSprite) As Long
    On Error GoTo blt_error
    Err.Clear
    If Sprite.TransparencyEnabled Then
        SpriteBltFast = DoubleBuffer.BltFast(Sprite.X, Sprite.Y, Sprite.Surface, Sprite.FrameRect(Sprite.CurrentFrame), DDBLTFAST_WAIT Or DDBLTFAST_SRCCOLORKEY)
    Else
        SpriteBltFast = DoubleBuffer.BltFast(Sprite.X, Sprite.Y, Sprite.Surface, Sprite.FrameRect(Sprite.CurrentFrame), DDBLTFAST_WAIT)
    End If
    Exit Function
blt_error:
    SpriteBltFast = Err.Number
End Function

Public Function SpriteBackground(ByRef Sprite As clsSprite) As Long
    On Error GoTo behind_error
    Err.Clear
    Dim BehindRect As DxVBLib.RECT
    Dim BehindDesc As DDSURFACEDESC2
    'First define the size of the background rect
    With Sprite
        Dim rWidth As Long, rHeight As Long
        rWidth = .FrameRect(.CurrentFrame).Right - .FrameRect(.CurrentFrame).Left
        rHeight = .FrameRect(.CurrentFrame).Bottom - .FrameRect(.CurrentFrame).Top
    End With
    With BehindRect
        .Left = Sprite.X
        .Top = Sprite.Y
        .Right = .Left + rWidth
        .Bottom = .Top + rHeight
    End With
    'Now resize the background surface
    With BehindDesc
        .lFlags = DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH
        .ddsCaps.lCaps = DDSCAPS_OFFSCREENPLAIN
        .lWidth = rWidth
        .lHeight = rHeight
    End With
    Set Sprite.BackgroundSurface = DirectDraw.CreateSurface(BehindDesc)
    If Sprite.BackgroundSurface Is Nothing Then GoTo behind_error
    'Now copy the background into memory
    SpriteBackground = Sprite.BackgroundSurface.BltFast(0, 0, DoubleBuffer, BehindRect, DDBLTFAST_WAIT)
    Exit Function
behind_error:
    SpriteBackground = Err.Number
End Function

Public Function SpriteErase(ByRef Sprite As clsSprite) As Long
    On Error GoTo erase_error
    Err.Clear
    With Sprite
        SpriteErase = DoubleBuffer.BltFast(.X, .Y, .BackgroundSurface, DXRect(0, 0, .FrameRect(.CurrentFrame).Right - .FrameRect(.CurrentFrame).Left, .FrameRect(.CurrentFrame).Bottom - .FrameRect(.CurrentFrame).Top), DDBLTFAST_WAIT)
    End With
    Exit Function
erase_error:
    SpriteErase = Err.Number
End Function

'Drawing surface blitting functions
Public Function DrawingSurfaceBlt(ByRef Surface As clsDrawingSurface) As Long
    On Error GoTo blt_error
    Err.Clear
    If Surface.TransparencyEnabled Then
        DrawingSurfaceBlt = DoubleBuffer.Blt(BlittingRect, Surface.Surface, Surface.Rectangle, DDBLT_KEYSRC Or DDBLT_WAIT)
    Else
        DrawingSurfaceBlt = DoubleBuffer.Blt(BlittingRect, Surface.Surface, Surface.Rectangle, DDBLT_WAIT)
    End If
    Exit Function
blt_error:
    DrawingSurfaceBlt = Err.Number
End Function

Public Function DrawingSurfaceBltFast(ByRef Surface As clsDrawingSurface, Optional X As Long = 0, Optional Y As Long = 0) As Long
    On Error GoTo blt_error
    Err.Clear
    If Surface.TransparencyEnabled Then
        DrawingSurfaceBltFast = DoubleBuffer.BltFast(X, Y, Surface.Surface, Surface.Rectangle, DDBLTFAST_SRCCOLORKEY Or DDBLTFAST_WAIT)
    Else
        DrawingSurfaceBltFast = DoubleBuffer.BltFast(X, Y, Surface.Surface, Surface.Rectangle, DDBLTFAST_WAIT)
    End If
    Exit Function
blt_error:
    DrawingSurfaceBltFast = Err.Number
End Function

Public Sub ResetRectangle()
    With BlittingRect
        .Top = 0
        .Left = 0
        .Right = DoubleBufferDesc.lWidth
        .Bottom = DoubleBufferDesc.lHeight
    End With
End Sub

Public Sub Reset()
    DirectDraw.RestoreAllSurfaces
End Sub

Public Function ColourARGB(ByVal Red As Single, ByVal Green As Single, ByVal Blue As Single, ByVal Alpha As Single) As Long
    ColourARGB = DirectX.CreateColorRGBA(Red, Green, Blue, Alpha)
End Function

Public Function ColourRGB(ByVal Red As Single, ByVal Green As Single, ByVal Blue As Single) As Long
    ColourRGB = DirectX.CreateColorRGB(Red, Green, Blue)
End Function

'Basic Drawing functions
Public Function DrawText(Text As String, X As Long, Y As Long) As Long
    On Error GoTo text_error
    Err.Clear
    DoubleBuffer.DrawText X, Y, Text, False
    DrawText = 0
    Exit Function
text_error:
    DrawText = Err.Number
End Function

Public Sub SetFont(TextFont As StdFont)
    Dim DispFont As IFont
    Set DispFont = TextFont
    DoubleBuffer.SetFont DispFont
End Sub

Public Function DrawLine(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long) As Long
    On Error GoTo line_error
    Err.Clear
    DoubleBuffer.DrawLine X1, Y1, X2, Y2
    DrawLine = 0
    Exit Function
line_error:
    DrawLine = Err.Number
End Function

Public Function DrawBox(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long) As Long
    On Error GoTo box_error
    Err.Clear
    DoubleBuffer.DrawBox X1, Y1, X2, Y2
    DrawBox = 0
    Exit Function
box_error:
    DrawBox = Err.Number
End Function

Public Function DrawRoundedBox(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long, EllipseWidth As Long, EllipseHeight As Long) As Long
    On Error GoTo roundbox_error
    Err.Clear
    DoubleBuffer.DrawRoundedBox X1, Y1, X2, Y2, EllipseWidth, EllipseHeight
    DrawRoundedBox = 0
    Exit Function
roundbox_error:
    DrawRoundedBox = Err.Number
End Function

Public Function DrawCircle(X As Long, Y As Long, Radius As Long) As Long
    On Error GoTo circle_error
    Err.Clear
    DoubleBuffer.DrawCircle X, Y, Radius
    DrawCircle = 0
    Exit Function
circle_error:
    DrawCircle = Err.Number
End Function

Public Function DrawEllipse(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long) As Long
    On Error GoTo ellipse_error
    Err.Clear
    DoubleBuffer.DrawEllipse X1, Y1, X2, Y2
    DrawEllipse = 0
    Exit Function
ellipse_error:
    DrawEllipse = Err.Number
End Function

'*************************
'Class Properties
'*************************
Public Property Get WindowInFocus() As Boolean
    If DirectDraw.TestCooperativeLevel() = DD_OK Then
        WindowInFocus = True
    Else
        WindowInFocus = False
    End If
End Property

Public Property Get Rectangle() As DxVBLib.RECT
    Rectangle = BlittingRect
End Property

Public Property Let Rectangle(r As DxVBLib.RECT)
    BlittingRect = r
End Property

Public Property Let ForeColour(Colour As Long)
    DoubleBuffer.SetForeColor Colour
End Property

Public Property Get ForeColour() As Long
    ForeColour = DoubleBuffer.GetForeColor
End Property

Public Property Let FillColour(Colour As Long)
    DoubleBuffer.SetFillColor Colour
End Property

Public Property Get FillColour() As Long
    FillColour = DoubleBuffer.GetFillColor
End Property

Public Property Let FillStyle(Style As FillStyleConstants)
    DoubleBuffer.SetFillStyle Style
End Property

Public Property Get FillStyle() As FillStyleConstants
    FillStyle = DoubleBuffer.GetFillStyle
End Property

Public Property Let DrawStyle(Style As DrawStyleConstants)
    DoubleBuffer.setDrawStyle Style
End Property

Public Property Get DrawStyle() As DrawStyleConstants
    DrawStyle = DoubleBuffer.getDrawStyle
End Property

Public Property Let DrawWidth(Width As Long)
    DoubleBuffer.setDrawWidth Width
End Property

Public Property Get DrawWidth() As Long
    DrawWidth = DoubleBuffer.getDrawWidth
End Property
