VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsSprite"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'********************************************
'* clsSprite.cls                            *
'* Version 1.1                              *
'* Full sprite handling facilities          *
'********************************************
Option Explicit

'*************************
'Global Enums and types
'*************************
Private Type SpriteFrame
    FrameRect As DxVBLib.RECT
    FrameBB As DxVBLib.RECT
End Type

Public Enum SpriteDirectionEnum
    SPRITE_DOWN = 0
    SPRITE_RIGHT = 1
    SPRITE_UP = 2
    SPRITE_LEFT = 3
    SPRITE_TOPLEFT = 4
    SPRITE_TOPRIGHT = 5
    SPRITE_BOTTOMLEFT = 6
    SPRITE_BOTTOMRIGHT = 7
End Enum

Public Enum SpriteStateEnum
    SPRITE_OK = 0
    SPRITE_DYING = 1
    SPRITE_DEAD = 2
End Enum

'Bitmap variables
Public SpriteSurface As DirectDrawSurface7 'Surface that holds ALL the frames!
Dim SurfaceDesc As DDSURFACEDESC2
Dim Background As DirectDrawSurface7
Dim BackgroundDesc As DDSURFACEDESC2
Dim ColourKey As DDCOLORKEY
Dim TransparentBlt As Boolean
Dim BitmapFilename As String

'Sprite Variables
Dim SpriteFrames() As SpriteFrame     'Array to hold the rects that point to specific frames and the bounding box rects
Dim SpriteX As Single, SpriteY As Single
Dim SpriteOldX As Single, SpriteOldY As Single
Dim SpriteState As SpriteStateEnum
Dim SpriteCurrFrame As Long
Dim SpriteHealth As Single
Dim SpriteAnimTimer As Single, SpriteAnimSpeed As Single
Dim SpriteMotionTimer As Single, SpriteMotionSpeed As Single
Dim SpriteWidth As Long, SpriteHeight As Long
Dim SpriteDirection As SpriteDirectionEnum
Dim SpriteNumFrames As Long

'*****************************
'Class initialize and terminate
'*****************************
Private Sub Class_Initialize()

End Sub

Private Sub Class_Terminate()
    Set SpriteSurface = Nothing
    Set Background = Nothing
    ReDim SpriteFrames(0)
End Sub

'****************************
'Class Methods
'****************************
Public Function Load(Filename As String, ByRef DirectDrawEngine As clsDirectDraw, Optional AutoFillFrames As Boolean = True) As Long
    On Error GoTo load_error
    Err.Clear
    
    'First load the bitmap into memory
    Set SpriteSurface = Nothing
    SurfaceDesc.lFlags = DDSD_CAPS
    SurfaceDesc.ddsCaps.lCaps = DDSCAPS_OFFSCREENPLAIN
    Set SpriteSurface = DirectDrawEngine.DirectDraw.CreateSurfaceFromFile(Filename, SurfaceDesc)
    If SpriteSurface Is Nothing Then GoTo load_error
    
    SpriteSurface.GetSurfaceDesc SurfaceDesc
    
    'Now check to see if the user wants the frame rects automatically computed
    If AutoFillFrames Then
        Dim MaxFramesWidth As Long, MaxFramesHeight As Long
        Dim FinishedFrames As Long
        Dim WidthCounter As Long, HeightCounter As Long
        Dim Top As Long, Left As Long
        'First compute the maximum number of frames
        'that can be fitted along the width as height of the bitmap
        MaxFramesWidth = Int(SurfaceDesc.lWidth / SpriteWidth)
        MaxFramesHeight = Int(SurfaceDesc.lHeight / SpriteHeight)
        'Now start setting the frame rects
        FinishedFrames = 0
        For HeightCounter = 0 To MaxFramesHeight - 1
            For WidthCounter = 0 To MaxFramesWidth - 1
                Left = WidthCounter * SpriteWidth
                Top = HeightCounter * SpriteHeight
                SpriteFrames(FinishedFrames).FrameRect = DXRect(Left, Top, Left + SpriteWidth, Top + SpriteHeight)
                FinishedFrames = FinishedFrames + 1
                If FinishedFrames = SpriteNumFrames Then GoTo end_autoframe
            Next WidthCounter
        Next HeightCounter
end_autoframe:
    End If
    
    BitmapFilename = Filename
    Load = 0
    Exit Function
load_error:
    Load = Err.Number
End Function

Public Sub SetAllBoundingBoxRects(BBRect As DxVBLib.RECT)
    On Error GoTo bb_error
    Dim Index As Long
    
    For Index = 0 To SpriteNumFrames - 1
        SpriteFrames(Index).FrameBB = BBRect
    Next Index
    Exit Sub
bb_error:
    Exit Sub
End Sub

Public Function Collide(ByRef Sprite As clsSprite) As Boolean
    On Error GoTo collide_error
    'Note all variables ending in 1 are reference to this sprite
    'and those ending with 2 are reference to the parameter sprite
    Dim MinX As Long, MinY As Long
    Dim MaxX As Long, MaxY As Long
    Dim BBWidth As Long, BBHeight As Long
    
    'Find out which is more to the right
    Dim ActualX1 As Long, ActualX2 As Long
    ActualX1 = SpriteX + SpriteFrames(SpriteCurrFrame).FrameBB.Left
    ActualX2 = Sprite.X + Sprite.FrameBoundingBox(Sprite.CurrentFrame).Left
    If ActualX1 > ActualX2 Then
        MaxX = ActualX1
        MinX = ActualX2
        BBWidth = Sprite.FrameBoundingBox(Sprite.CurrentFrame).Right - Sprite.FrameBoundingBox(Sprite.CurrentFrame).Left
    Else
        MaxX = ActualX2
        MinX = ActualX1
        BBWidth = SpriteFrames(SpriteCurrFrame).FrameBB.Right - SpriteFrames(SpriteCurrFrame).FrameBB.Left
    End If
    
    'Now find out which is lower down
    Dim ActualY1 As Long, ActualY2 As Long
    ActualY1 = SpriteY + SpriteFrames(SpriteCurrFrame).FrameBB.Top
    ActualY2 = Sprite.Y + Sprite.FrameBoundingBox(Sprite.CurrentFrame).Top
    If ActualY1 > ActualY2 Then
        MaxY = ActualY1
        MinY = ActualY2
        BBHeight = Sprite.FrameBoundingBox(Sprite.CurrentFrame).Bottom - Sprite.FrameBoundingBox(Sprite.CurrentFrame).Top
    Else
        MaxY = ActualY2
        MinY = ActualY1
        BBHeight = SpriteFrames(SpriteCurrFrame).FrameBB.Bottom - SpriteFrames(SpriteCurrFrame).FrameBB.Top
    End If
    
    'Now do the collision testing
    If ((MaxX >= MinX) And (MaxX < (MinX + BBWidth))) And ((MaxY >= MinY) And (MaxY < (MinY + BBHeight))) Then
        Collide = True
    Else
        Collide = False
    End If
    Exit Function
collide_error:
    Collide = False
End Function

Public Function Reset(ByRef DirectDrawEngine As clsDirectDraw) As Long
    On Error GoTo reset_error
    Err.Clear
    Set SpriteSurface = DirectDrawEngine.DirectDraw.CreateSurfaceFromFile(BitmapFilename, SurfaceDesc)
    Reset = 0
    Exit Function
reset_error:
    Reset = Err.Number
End Function

'****************************
'Class Properties
'****************************
Public Property Let X(XPos As Single)
    SpriteX = XPos
End Property

Public Property Get X() As Single
    X = SpriteX
End Property

Public Property Let Y(YPos As Single)
    SpriteY = YPos
End Property

Public Property Get Y() As Single
    Y = SpriteY
End Property

Public Property Let OldX(XPos As Single)
    SpriteOldX = XPos
End Property

Public Property Get OldX() As Single
    OldX = SpriteOldX
End Property

Public Property Let OldY(YPos As Single)
    SpriteOldY = YPos
End Property

Public Property Get OldY() As Single
    OldY = SpriteOldY
End Property

Public Property Let State(S As SpriteStateEnum)
    SpriteState = S
End Property

Public Property Get State() As SpriteStateEnum
    State = SpriteState
End Property

Public Property Let CurrentFrame(CurrFrame As Long)
    SpriteCurrFrame = CurrFrame
End Property

Public Property Get CurrentFrame() As Long
    CurrentFrame = SpriteCurrFrame
End Property

Public Property Let Health(H As Single)
    SpriteHealth = H
End Property

Public Property Get Health() As Single
    Health = SpriteHealth
End Property

Public Property Let AnimationTimer(AnimTimer As Single)
    SpriteAnimTimer = AnimTimer
End Property

Public Property Get AnimationTimer() As Single
    AnimationTimer = SpriteAnimTimer
End Property

Public Property Let AnimationSpeed(AnimSpeed As Single)
    SpriteAnimSpeed = AnimSpeed
End Property

Public Property Get AnimationSpeed() As Single
    AnimationSpeed = SpriteAnimSpeed
End Property

Public Property Let MotionTimer(MotionTimer As Single)
    SpriteMotionTimer = MotionTimer
End Property

Public Property Get MotionTimer() As Single
    MotionTimer = SpriteMotionTimer
End Property

Public Property Let MotionSpeed(MotionSpeed As Single)
    SpriteMotionSpeed = MotionSpeed
End Property

Public Property Get MotionSpeed() As Single
    MotionSpeed = SpriteMotionSpeed
End Property

Public Property Let Width(W As Long)
    SpriteWidth = W
End Property

Public Property Get Width() As Long
    Width = SpriteWidth
End Property

Public Property Let Height(H As Long)
    SpriteHeight = H
End Property

Public Property Get Height() As Long
    Height = SpriteHeight
End Property

Public Property Let Direction(D As SpriteDirectionEnum)
    SpriteDirection = D
End Property

Public Property Get Direction() As SpriteDirectionEnum
    Direction = SpriteDirection
End Property

Public Property Let FrameRect(Index As Long, Rectangle As DxVBLib.RECT)
    On Error Resume Next
    SpriteFrames(Index).FrameRect = Rectangle
End Property

Public Property Get FrameRect(Index As Long) As DxVBLib.RECT
    On Error Resume Next
    FrameRect = SpriteFrames(Index).FrameRect
End Property

Public Property Let FrameBoundingBox(Index As Long, Rectangle As DxVBLib.RECT)
    On Error Resume Next
    SpriteFrames(Index).FrameBB = Rectangle
End Property

Public Property Get FrameBoundingBox(Index As Long) As DxVBLib.RECT
    On Error Resume Next
    FrameBoundingBox = SpriteFrames(Index).FrameBB
End Property

Public Property Get BitmapWidth() As Long
    BitmapWidth = SurfaceDesc.lWidth
End Property

Public Property Get BitmapHeight() As Long
    BitmapHeight = SurfaceDesc.lHeight
End Property

Public Property Get Surface() As DirectDrawSurface7
    Set Surface = SpriteSurface
End Property

Public Property Let NumFrames(Num As Long)
    On Error Resume Next
    SpriteNumFrames = Num
    ReDim Preserve SpriteFrames(Num)
End Property

Public Property Get NumFrames() As Long
    NumFrames = SpriteNumFrames
End Property

Public Property Get BackgroundSurface() As DirectDrawSurface7
    Set BackgroundSurface = Background
End Property

Public Property Set BackgroundSurface(S As DirectDrawSurface7)
    Set Background = S
End Property

Public Property Let TransparentColour(Col As DDCOLORKEY)
    ColourKey = Col
    SpriteSurface.SetColorKey DDCKEY_SRCBLT, Col
End Property

Public Property Get TransparentColour() As DDCOLORKEY
    TransparentColour = ColourKey
End Property

Public Property Let TransparencyEnabled(T As Boolean)
    TransparentBlt = T
End Property

Public Property Get TransparencyEnabled() As Boolean
    TransparencyEnabled = TransparentBlt
End Property
